(* ===================================================================

   The top level translator, represented as a functor

=================================================================== *)


local 

open Parse HolKernel boolLib 

open Option PP Err 
     L0Theory L0_emb_mldata L0_emb_basic 
     Translator 

in

functor build_TR (Tspec : Translator) =

   struct

   local

   fun translate progdef_thm pps =

        let

        val prg = (buildPROG
                        o concl
                        o SPEC_ALL) 
                          progdef_thm


       in

       ( clear_ppstream pps
       ; Tspec.translate_prog (#pname prg)
                              (#pparams prg)
                              (#prparams prg)
                              (#pvparams prg)
                              (#preturn prg)
                              (fold_INSTR Tspec.tAlgebra (buildINSTRtree (#pbody prg)))
                               pps
       ; flush_ppstream pps )

       end
  
       handle L0Exn e => (printL0Error e; raise (L0Exn e))

   in

   (* producing the translation as a string *)

   fun toString progdef_thm = 
          let
          fun t pps def = translate def pps
          in
          pp_to_string 100 t progdef_thm
          end

   (* just printing the translation to the standard output *)

   fun print progdef_thm = BasicIO.print (toString progdef_thm)

   (* translate and save it to a file *)

   fun generate progdef_thm fname 
       =
       let 
       val f = TextIO.openOut fname
       in
       ( TextIO.output(f, toString progdef_thm) ; TextIO.closeOut f )
       end 

   end (* of inner local *)
   end (* of struct *)

end